import datetime
import pickle

from gridfs import GridFS
from pymongo import MongoClient

from promethist.common.interfaces.storage import Storage


class MongoDBStorage(Storage):
    def __init__(self, config):
        super().__init__(config)
        self.client = MongoClient(config["mongodbUrl"])
        self.db_name = config["name"] + "-" + config["namespace"]
        self.db = self.client[self.db_name]
        self.grid_fs = GridFS(self.db)
        self.collection = self.db['model']

    def get_model(self, _id, language, newer_than=datetime.datetime.min):
        data = self.collection.find_one({'_id': _id, 'language': language, 'lastModified': {'$gt': newer_than}})
        data = pickle.loads(data["data"])
        return data["class_name"], data["model"], data["mapping"], data["config"]

    def save_model(self, _id, language, data):
        self.collection.update_one({'_id': _id, 'language': language}, {"$set": {"data": data, "lastModified": datetime.datetime.now()}}, upsert=True)

    def delete_model(self, _id, language):
        self.collection.delete_one({'_id': _id, 'language': language})

